/************************************************************************
*
* Device     : RX/RX600/RX63N,RX631
*
* File Name  : hwsetup.c
*
* Abstract   : Hardware Setup file.
*
* History    : 0.10  (2011-02-21)  [Hardware Manual Revision : 0.01]
*
* NOTE       : THIS IS A TYPICAL EXAMPLE.
*
* Copyright (C) 2011 Renesas Electronics Corporation.
* and Renesas Solutions Corp.
*
************************************************************************/

#include "platform.h"
#include "r_cg_macrodriver.h"
#ifdef __cplusplus
extern "C" {
#endif
extern void HardwareSetup(void);
extern void ConfigureOperatingFrequency(void);
#ifdef __cplusplus
}
#endif

void HardwareSetup(void)
{
    R_Systeminit();
}

/*
 * RX63N CubeSuite+ Renesas Starter Kit+ Sample Code
 * https://www.renesas.com/jp/ja/software/D3013175.html
 *
 * an_r20an0272jj0100_rx63n_rsk.zip/an_r20an0272jj0100_rx63n_rsk/63nrsk/Samples/Async_Serial/
 * hwsetup.c
 */

void ConfigureOperatingFrequency(void)
{
    /* Protection off */
    SYSTEM.PRCR.WORD = 0xA503u;

    /* Stop sub-clock */
    SYSTEM.SOSCCR.BYTE = 0x01u;

    /* Set main oscillator settling time to 10ms (131072 cycles @ 12MHz) */
    SYSTEM.MOSCWTCR.BYTE = 0x0Du;

    /* Set PLL circuit settling time to 10ms (2097152 cycles @ 192MHz) */
    SYSTEM.PLLWTCR.BYTE = 0x0Eu;

    /* Set PLL circuit to x16 */
    SYSTEM.PLLCR.WORD = 0x0F00u;

    /* Start the external 12Mhz oscillator */
    SYSTEM.MOSCCR.BYTE = 0x00u;

    /* Turn on the PLL */
    SYSTEM.PLLCR2.BYTE = 0x00u;

    /* Wait over 12ms (~2075op/s @ 125KHz) */
    for(volatile uint16_t i = 0; i < 2075u; i++)
    {
        nop();
    }

    /* Configure the clocks as follows -
    Clock Description              Frequency
    ----------------------------------------
    PLL Clock frequency...............192MHz
    System Clock Frequency.............96MHz
    Peripheral Module Clock B..........48MHz
    FlashIF Clock......................48MHz
    External Bus Clock.................48MHz */
    SYSTEM.SCKCR.LONG = 0x21821211u;

    /* Configure the clocks as follows -
    Clock Description              Frequency
    ----------------------------------------
    USB Clock..........................48MHz
    IEBus Clock........................24MHz */
    SYSTEM.SCKCR2.WORD = 0x0033u;

    /* Set the clock source to PLL */
    SYSTEM.SCKCR3.WORD = 0x0400u;

    /* Protection on */
    SYSTEM.PRCR.WORD = 0xA500u;
}
